package com.netiq.nam.oauth.helpers;

import com.google.api.client.auth.oauth2.BearerToken;
import com.google.api.client.auth.oauth2.Credential;
import com.google.api.client.auth.oauth2.StoredCredential;
import com.google.api.client.http.GenericUrl;
import com.google.api.client.http.HttpRequestFactory;
import com.google.api.client.http.HttpResponse;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.http.json.JsonHttpContent;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.api.client.json.jackson2.JacksonFactory;
import com.google.api.client.util.store.FileDataStoreFactory;

import javax.net.ssl.*;
import java.io.File;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Created by tsureshkumar on 12/7/14.
 */
public class Http {
    public static final NetHttpTransport INSECURE_TRANSPORT = insecureHttpTransport();
    public static final JsonFactory JSON_FACTORY = new JacksonFactory();

    /*
    * uses insecure trust all https transport
     */
    public HttpResponse insecureGet(String url, String oauth2Token) throws IOException {
        Logger logger = Logger.getLogger(HttpTransport.class.getName());
        logger.setLevel(Level.FINEST);
        Credential credential =
                new Credential(BearerToken.authorizationHeaderAccessMethod()).setAccessToken(oauth2Token);
        HttpRequestFactory requestFactory = INSECURE_TRANSPORT.createRequestFactory(credential);
        return requestFactory
                .buildGetRequest(new GenericUrl(url))
                .setLoggingEnabled(true)
                .setCurlLoggingEnabled(true)
                .setParser(new JsonObjectParser(JSON_FACTORY))
                .execute();
    }

    public HttpResponse insecurePost(String url, String accessToken, Object jsonData) throws IOException {
        Credential credential =
                new Credential(BearerToken.authorizationHeaderAccessMethod()).setAccessToken(accessToken);
        HttpRequestFactory requestFactory = INSECURE_TRANSPORT.createRequestFactory(credential);
        return requestFactory
                .buildPostRequest(new GenericUrl(url),
                        new JsonHttpContent(JSON_FACTORY, jsonData))
                .setLoggingEnabled(true)
                .setCurlLoggingEnabled(true)
                .setParser(new JsonObjectParser(JSON_FACTORY))
                .execute();
    }

    /*
    * This trusts all certificates of the server url during https ssl verification
    * This should not be used in production.  Instead, add the trusted authorities to your java's truststore
    * and invoke the server with -Djavax.net.ssl.trustStore=path/to/keystore.jks
     */
    public static NetHttpTransport insecureHttpTransport() {
        SSLContext sslContext = null;
        try {
            sslContext = SSLContext.getInstance("SSL");
            sslContext.init(null, new TrustManager[]{new X509TrustManager() {
                public X509Certificate[] getAcceptedIssuers() {
                    return null;
                }

                public void checkClientTrusted(X509Certificate[] certs,
                                               String authType) {
                }

                public void checkServerTrusted(X509Certificate[] certs,
                                               String authType) {
                }
            }}, new SecureRandom());


            return new NetHttpTransport.Builder()
                    .setSslSocketFactory(sslContext.getSocketFactory())
                    .setHostnameVerifier(new HostnameVerifier() {
                        @Override
                        public boolean verify(String s, SSLSession sslSession) {
                            return true;
                        }
                    })
                    .build();
        } catch (KeyManagementException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e1) {
            e1.printStackTrace();
        }
        return new NetHttpTransport();
    }


}
