/*
* This sample uses extensively google-oauth2-client-java project. https://code.google.com/p/google-oauth-java-client/
* code adopted from com.google.api.client.extensions.servlet.auth.oauth2.AbstractAuthorizationCodeServlet
 */
package com.netiq.nam.oauth.helpers;

import com.google.api.client.auth.oauth2.*;
import com.google.api.client.http.GenericUrl;
import com.google.api.client.http.HttpResponseException;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

public class OAuth2Filter implements Filter {
    private final Lock lock = new ReentrantLock();


    private Credential credential;
    private AuthorizationCodeFlow flow;

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {

    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
        HttpServletRequest req = (HttpServletRequest) servletRequest;
        HttpServletResponse resp = (HttpServletResponse) servletResponse;

        lock.lock();
        try {
            // load credential from persistence store
            String userId = getUserId(req);
            if (flow == null) {
                flow = initializeFlow();
            }
            credential = flow.loadCredential(userId);
            // if credential found with an access token, invoke the user code
            if (credential != null && credential.getAccessToken() != null) {
                try {
                    filterChain.doFilter(req, resp);
                    return;
                } catch (HttpResponseException e) {
                    // if access token is null, assume it is because auth failed and we need to re-authorize
                    // but if access token is not null, it is some other problem
                    if (credential.getAccessToken() != null) {
                        throw e;
                    }
                }
            }
            // redirect to the authorization flow
            AuthorizationCodeRequestUrl authorizationUrl = flow.newAuthorizationUrl();
            authorizationUrl.setRedirectUri(getRedirectUri(req));
            onAuthorization(req, resp, authorizationUrl);
            credential = null;
        } finally {
            lock.unlock();
        }
    }

    @Override
    public void destroy() {

    }

    protected AuthorizationCodeFlow initializeFlow() throws ServletException, IOException {
        return new AuthorizationCodeFlow.Builder(BearerToken.authorizationHeaderAccessMethod(),
                Http.INSECURE_TRANSPORT,
                Http.JSON_FACTORY,
                new GenericUrl(OAuth2Properties.tokenEndPoint()),
                new ClientParametersAuthentication(OAuth2Properties.clientID(), OAuth2Properties.clientSecret()),
                OAuth2Properties.clientID(),
                OAuth2Properties.authorizationEndpoint())
                .setCredentialDataStore(OAuth2Helper.DATASTORE)
                .build();
    }

    protected String getRedirectUri(HttpServletRequest req) throws ServletException, IOException {
        GenericUrl url = new GenericUrl(req.getRequestURL().toString());
        url.setRawPath("/oauth2callback");
        return url.build();
    }

    protected String getUserId(HttpServletRequest req) throws ServletException, IOException {
        return (String) req.getSession(true).getId();
    }

    protected void onAuthorization(HttpServletRequest req, HttpServletResponse resp,
                                   AuthorizationCodeRequestUrl authorizationUrl) throws ServletException, IOException {
        resp.sendRedirect(authorizationUrl.build());
    }
}
