package com.netiq.nam.oauth.authz_sample.app;

import com.google.api.client.util.ArrayMap;
import com.netiq.nam.oauth.helpers.Http;
import com.netiq.nam.oauth.helpers.OAuth2Helper;
import com.netiq.nam.oauth.servlets.OAuth2ProtectedServlet;

import javax.servlet.ServletException;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/*
 * This servlet displays list of tasks by contacting back-end web service running on    "https://localhost:10443/rest/todos"
 * by passing OAuth2 token received as part of Authorization code flow
 *
 * This servlet derives from OAuth2ProtectedServlet which makes sure that before invoking this servlet, OAuth2 flow is setup
 * and a token is available.
 */
public class TasksServlet extends OAuth2ProtectedServlet {
    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String id = req.getParameter("id");
        String description = req.getParameter("description");

        String accessToken = OAuth2Helper.getAccessToken(getUserId(req));

        // Use the access token to execute the back-end rest service. Here in this case, it tries to get the list of
        // tasks
        ArrayList<ArrayMap<String, String>> jsonData = new ArrayList<>();
        ArrayMap<String, String> taskMap = new ArrayMap<>();
        taskMap.put("id", id);
        taskMap.put("description", description);
        jsonData.add(taskMap);
        List<ArrayMap<String, String>> tasks = (List) new Http().insecurePost("https://localhost:10443/rest/todos", accessToken,
                jsonData).parseAs(List.class);
         resp.sendRedirect("/tasks");

    }

    /*
     * before executing doGet,the    AbstractAuthorizationCodeServlet checks whether the current session has been
     * authenticated with OAuth and token is availble.  If not, it redirects to Authorization server and exchanges
     * OAuth flows to get access token using Authorization code flow.
      */
    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        try {
            // Get the cached token stored in session or data store
            String accessToken = OAuth2Helper.getAccessToken(getUserId(req));

            // Use the access token to execute the back-end rest service. Here in this case, it tries to get the list of
            // tasks
            List<ArrayMap<String, String>> tasks = (List) new Http().insecureGet("https://localhost:10443/rest/todos", accessToken).parseAs(List.class);

            // print the tasks
            PrintWriter writer = resp.getWriter();
            writer.write("<html>");
            printHeader(writer);
            writer.write("<body>");
            printMenu(writer, req, resp);
            writer.write("<ul>");
            tasks.stream().forEach(x -> {
                writer.write("<li><b>" + x.get("id") + "</b><div>" + x.get("description") + "</div></li>");
            });
            writer.write("</ul>");
            writer.write("</body></html>");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void printHeader(PrintWriter writer) {
        writer.write("<head><link rel=\"stylesheet\" type=\"text/css\" href=\"css/main.css\"></head>");
    }

    private void printMenu(PrintWriter writer, HttpServletRequest req, ServletResponse resp) throws ServletException, IOException {
       req.getRequestDispatcher("/menu.jsp").include(req, resp);
    }

}
