package com.netiq.nam.oauth.authz_sample.app.debug;

import com.google.api.client.auth.oauth2.AuthorizationCodeFlow;
import com.google.api.client.auth.oauth2.AuthorizationCodeResponseUrl;
import com.google.api.client.auth.oauth2.Credential;
import com.google.api.client.http.GenericUrl;
import com.netiq.nam.oauth.helpers.Http;
import com.netiq.nam.oauth.helpers.OAuth2Helper;
import com.netiq.nam.oauth.helpers.OAuth2Properties;
import com.netiq.nam.oauth.helpers.Token;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * Created by tsureshkumar on 12/7/14.
 */
public class AuthorizationCallbackStepwise extends HttpServlet {

    public static final String USER_ID = "USER_ID";
    public static final String OAUTH_2_TOKEN = "OAUTH2_TOKEN";

    protected void onSuccess(HttpServletRequest req, HttpServletResponse resp, Credential credential)
            throws ServletException, IOException {
        Token token = new Http().insecureGet(OAuth2Properties.tokenInfoEndpoint(), credential.getAccessToken()).parseAs(Token.class);
        if (token != null) {
            req.getSession(true).setAttribute(USER_ID, token.getUser());
            req.getSession(true).setAttribute(OAUTH_2_TOKEN, token);
        } else {
            resp.sendError(500);
            resp.getWriter().write("error getting token info");
        }
        resp.sendRedirect("/");
    }

    protected void onError(
            HttpServletRequest req, HttpServletResponse resp, AuthorizationCodeResponseUrl errorResponse)
            throws ServletException, IOException {
        resp.sendError(500);
        resp.getWriter().write("error getting token");
    }

    protected String getRedirectUri(HttpServletRequest req) throws ServletException, IOException {
        GenericUrl url = new GenericUrl(req.getRequestURL().toString());
        url.setRawPath("/oauth2callback_step");
        return url.build();
    }

    protected AuthorizationCodeFlow initializeFlow() throws IOException {
        return OAuth2Helper.getInstance().getFlow();
    }

    protected String getUserId(HttpServletRequest req) throws ServletException, IOException {
        return (String) req.getSession(true).getId();
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        StringBuffer buf = req.getRequestURL();
        if (req.getQueryString() != null) {
            buf.append('?').append(req.getQueryString());
        }
        AuthorizationCodeResponseUrl responseUrl = new AuthorizationCodeResponseUrl(buf.toString());
        String code = responseUrl.getCode();
        if (responseUrl.getError() != null) {
            onError(req, resp, responseUrl);
        } else if (code == null) {
            resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            resp.getWriter().print("Missing authorization code");
        } else {
            req.getRequestDispatcher("/code.jsp").include(req, resp);
        }

    }
}