package com.netiq.nam.oauth.authz_sample.app.debug;

import com.google.api.client.auth.oauth2.AuthorizationCodeFlow;
import com.google.api.client.auth.oauth2.AuthorizationCodeResponseUrl;
import com.google.api.client.auth.oauth2.Credential;
import com.google.api.client.auth.oauth2.TokenResponse;
import com.netiq.nam.oauth.helpers.OAuth2Helper;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Created by tsureshkumar on 12/7/14.
 */
public class AuthorizationCallbackStepwise_2 extends AuthorizationCallbackStepwise {
    private final Lock lock = new ReentrantLock();

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        StringBuffer buf = req.getRequestURL();
        if (req.getQueryString() != null) {
            buf.append('?').append(req.getQueryString());
        }
        AuthorizationCodeResponseUrl responseUrl = new AuthorizationCodeResponseUrl(buf.toString());
        String code = responseUrl.getCode();
        if (responseUrl.getError() != null) {
            onError(req, resp, responseUrl);
        } else if (code == null) {
            resp.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            resp.getWriter().print("Missing authorization code");
        } else {
            lock.lock();
            try {
                AuthorizationCodeFlow flow = OAuth2Helper.getInstance().getFlow();
                String redirectUri = getRedirectUri(req);
                TokenResponse response = flow.newTokenRequest(code).setRedirectUri(redirectUri).execute();
                String userId = getUserId(req);
                Credential credential = flow.createAndStoreCredential(response, userId);
                onSuccess(req, resp, credential);
            } finally {
                lock.unlock();
            }
        }
    }

    @Override
    protected void onSuccess(HttpServletRequest req, HttpServletResponse resp, Credential credential)
            throws ServletException, IOException {
        req.setAttribute("token", credential.getAccessToken());
        req.getRequestDispatcher("/token.jsp").include(req, resp);
    }
}